@@ -238,12 +238,6 @@ def wx_selected_wrapper(self, target):
     return wrapper
 
 
-def wx_buttonselected_wrapper(self, target):
-    def wrapper(event):
-        target(self.current_index())
-    return wrapper
-
-
 def wx_listviewselected_wrapper(self, target):
     def wrapper(event):
         # wx doesn't use the "list of lists" model so we have to use our current item method
@@ -292,25 +286,34 @@ wrapper_map = {
     SIGNAL_CLICKED: wx_plain_wrapper,
     SIGNAL_TOGGLED: wx_toggled_wrapper,
     SIGNAL_SELECTED: wx_selected_wrapper,
-    SIGNAL_BUTTONSELECTED: wx_buttonselected_wrapper,
     SIGNAL_FOCUS_IN: wx_plain_wrapper,
     SIGNAL_FOCUS_OUT: wx_plain_wrapper,
     SIGNAL_LISTVIEWSELECTED: wx_listviewselected_wrapper,
     SIGNAL_LISTBOXSELECTED: wx_listboxselected_wrapper,
-    SIGNAL_CELLSELECTED: wx_cellselected_wrapper,
-    SIGNAL_CELLCHANGED: wx_cellchanged_wrapper,
     SIGNAL_TEXTCHANGED: wx_plain_wrapper,
-    SIGNAL_TEXTMODCHANGED: wx_textmodchanged_wrapper,
-    SIGNAL_TEXTSTATECHANGED: wx_plain_wrapper,
     SIGNAL_EDITCHANGED: wx_editchanged_wrapper,
     SIGNAL_ENTER: wx_plain_wrapper,
     SIGNAL_TABSELECTED: wx_tabselected_wrapper,
-    SIGNAL_QUERYCLOSE: wx_plain_wrapper,
-    SIGNAL_CLOSING: wx_plain_wrapper,
-    SIGNAL_SHOWN: wx_plain_wrapper,
-    SIGNAL_HIDDEN: wx_plain_wrapper
 }
 
+no_wrapper_events = set([
+    SIGNAL_TABLECELLSELECTED,
+    SIGNAL_TABLECELLEDITING,
+    SIGNAL_TABLECELLEDITDONE,
+    SIGNAL_TABLECHANGED,
+    SIGNAL_SIZEEVENT,
+    SIGNAL_SHOWEVENT,
+    SIGNAL_MINIMIZEEVENT,
+    SIGNAL_CLOSEEVENT,
+])
+
+
+def args_wrapper(target):
+    def wrapper(event):
+        target(*event.args)
+    return wrapper
+
+
 # This event is for manual do_notify, since we can't instantiate wx.Event itself
 
 NotifyEvent, _EVT_NOTIFY = NewEvent()
@@ -330,8 +333,15 @@ class PWxSignal(object):
     def wrap_target(self, signal, target):
         target = super(PWxSignal, self).wrap_target(signal, target)
         if signal in wrapper_map:
+            # These events come from the toolkit internals and aren't used with do_notify, so the parameters
+            # the event handler expects need to be extracted from the wx Event object by the wrapper
             return wrapper_map[signal](self, target)
-        return target
+        if signal in no_wrapper_events:
+            # These events are handled internally by a widget, so the handler takes the event parameter
+            # directly; these events cannot be used with do_notify
+            return target
+        # These events are used with do_notify in user code and need the do_notify args
+        return args_wrapper(target)
     
     def connect_target(self, signal, target):
         if signal in event_map:
@@ -339,23 +349,20 @@ class PWxSignal(object):
             self.wx_register_event(event, target)
     
     def do_notify(self, signal, *args):
-        # FIXME: any way to pass on args in wx?
         if signal in custom_map:
-            event = custom_map[signal]()
+            event = custom_map[signal](args=args)
         else:
             assert signal in event_map
             # This is a kludge because wx doesn't allow you to instantiate wx.Event
             # and then set a type, and it doesn't give an easy way to instantiate
             # all the necessary specific event types
-            event = NotifyEvent()
+            event = NotifyEvent(args=args)
             event.SetEventType(event_map[signal]._getEvtType())
         if event is not None:
             if self._forward:
-                forward_event, fields = self._forward
+                forward_event = self._forward
                 event.SetId(forward_event.GetId())
                 event.SetEventObject(forward_event.GetEventObject())
-                for attr, value in (fields or ()):
-                    setattr(event, attr, value)
                 self._forward = None
             else:
                 if hasattr(self, 'id'):
@@ -367,10 +374,10 @@ class PWxSignal(object):
                 event.SetEventObject(self)
             self.AddPendingEvent(event)
     
-    def forward_event(self, signal, event, fields, *args):
+    def forward_event(self, signal, event, *args):
         # Ugly hack to allow data to be transferred to custom events
         # that are "translations" of built-in ones
-        self._forward = (event, fields)
+        self._forward = event
         self.do_notify(signal, *args)
 
 
@@ -696,9 +703,9 @@ class PTopWindow(PWxWidget, wx.Frame, PTopWindowBase):
     def SignalShown(self, event, shown):
         # 'automagic' method for SIGNAL_SHOWEVENT
         if shown:
-            self.forward_event(SIGNAL_SHOWN, event, None)
+            self.forward_event(SIGNAL_SHOWN, event)
         else:
-            self.forward_event(SIGNAL_HIDDEN, event, None)
+            self.forward_event(SIGNAL_HIDDEN, event)
     
     def OnShowEvent(self, event):
         # 'automagic' method for SIGNAL_SHOWEVENT
@@ -710,14 +717,14 @@ class PTopWindow(PWxWidget, wx.Frame, PTopWindowBase):
     
     def OnCloseWindow(self, event):
         # 'automagic' method for SIGNAL_CLOSEEVENT
-        self.forward_event(SIGNAL_QUERYCLOSE, event, None)  # a handler here can adjust state to be checked below
+        self.forward_event(SIGNAL_QUERYCLOSE, event)  # a handler here can adjust state to be checked below
         self.ProcessPendingEvents()  # force the handler to run before we check
         
         if event.CanVeto() and not self.can_close():
             event.Veto()
         else:
             # Send the closing signal
-            self.forward_event(SIGNAL_CLOSING, event, None)
+            self.forward_event(SIGNAL_CLOSING, event)
             # Allow signal handlers to process before destroying
             self.DestroyLater()
 
